/*
 * Copyright (c) 2009, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef CAMERA_FOCUS_DISPLAY_H
#define CAMERA_FOCUS_DISPLAY_H

#include <QWidget>

#include <rviz/display.h>

#include <pr2_object_manipulation_msgs/CameraFocus.h>

namespace Ui
{
class CameraFocus;
}

namespace rviz {
class VisualizationManager;
class WindowManagerInterface;
class PanelDockWidget;
}

namespace pr2_interactive_manipulation
{

class CameraFocusFrame;

class CameraFocusDisplay : public rviz::Display
{
public:
  CameraFocusDisplay();
  
  virtual ~CameraFocusDisplay();

protected:

  virtual void onEnable();

  virtual void onDisable();

  CameraFocusFrame *frame_; // The gui frame.
  rviz::PanelDockWidget* frame_dock_; // The dock which holds the gui frame.
};

class CameraFocusFrame : public QWidget
{
Q_OBJECT
public:
  CameraFocusFrame( rviz::VisualizationManager* manager, QWidget* parent = 0 );
  ~CameraFocusFrame();

protected Q_SLOTS:
  void leftButtonClicked();
  void topButtonClicked();
  void frontButtonClicked();
  void rightButtonClicked();
  //void acceptBoxToggled(bool checked);
  void overheadButtonClicked();
  void facingButtonClicked();

protected:
  void setCamera(std::vector<double> params);
  
  void callback(const pr2_object_manipulation_msgs::CameraFocusConstPtr &msg);

  ros::NodeHandle root_nh_;
  
  rviz::VisualizationManager* vis_manager_;

  ros::Subscriber sub_;

private:
  Ui::CameraFocus* ui_;
};

}

#endif
