/*
 * Copyright (c) 2011, Jeannette Bohg and Mårten Björkman 
 * ({bohg,celle}@csc.kth.se) 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *  1.Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  2.Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.  
 *  3.The name of Jeannette Bohg or Mårten Björkman may not be used to 
 *    endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <visualization_msgs/Marker.h>
#include <geometry_msgs/Point32.h>

namespace fast_plane_detection {
  
  visualization_msgs::Marker 
    getPlaneMarker( geometry_msgs::Point32 top_left,
		    geometry_msgs::Point32 top_right,
		    geometry_msgs::Point32 bottom_left,
		    geometry_msgs::Point32 bottom_right)
    {
      visualization_msgs::Marker marker;
      marker.action = visualization_msgs::Marker::ADD;
      marker.type = visualization_msgs::Marker::LINE_STRIP;
      marker.lifetime = ros::Duration();

      // create the marker in the table reference frame
      // the caller is responsible for setting the pose of 
      // the marker to match

      marker.scale.x = 0.001;
      marker.scale.y = 0.001;
      marker.scale.z = 0.001;

      marker.points.resize(5);
      marker.points[0].x = top_left.x;
      marker.points[0].y = top_left.y;
      marker.points[0].z = 0;
  
      marker.points[1].x = top_right.x;
      marker.points[1].y = top_right.y;
      marker.points[1].z = 0;
      
      marker.points[2].x = bottom_right.x;
      marker.points[2].y = bottom_right.y;
      marker.points[2].z = 0;
      
      marker.points[3].x = bottom_left.x;
      marker.points[3].y = bottom_left.y;
      marker.points[3].z = 0;
      
      marker.points[4].x = top_left.x;
      marker.points[4].y = top_left.y;
      marker.points[4].z = 0;

      marker.points.resize(6);
      marker.points[5].x = top_left.x;
      marker.points[5].y = top_left.y;
      marker.points[5].z = 0.02;
   
      marker.color.r = 1.0;
      marker.color.g = 1.0;
      marker.color.b = 0.0;
      marker.color.a = 1.0;

      return marker;
    }

}
