/*
 * Copyright (c) 2009, Willow Garage, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Willow Garage, Inc. nor the names of its
 *       contributors may be used to endorse or promote products derived from
 *       this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "object_recognition_gui/object_recognition_display.h"
#include "object_recognition_gui/object_recognition_rviz_ui.h"

#include <rviz/properties/property.h>
#include <rviz/properties/property_manager.h>

namespace object_recognition_gui {

ObjectRecognitionDisplay::ObjectRecognitionDisplay()
  : Display()
  , object_recognition_ui_(0)
{
}

ObjectRecognitionDisplay::~ObjectRecognitionDisplay()
{
  delete object_recognition_ui_;
}

void ObjectRecognitionDisplay::onEnable()
{
  object_recognition_ui_->startActionServer( update_nh_ );
}

void ObjectRecognitionDisplay::onDisable()
{
  object_recognition_ui_->stopActionServer();
}

void ObjectRecognitionDisplay::update(float wall_dt, float ros_dt)
{
  object_recognition_ui_->update(wall_dt, ros_dt);
}

void ObjectRecognitionDisplay::createProperties()
{
  if ( !object_recognition_ui_ )
  {
    ROS_INFO( "Creating UI" );
    object_recognition_ui_ = new ObjectRecognitionRvizUI(vis_manager_);
  }

  rviz::BoolPropertyWPtr lPropertyW = property_manager_->createProperty<rviz::BoolProperty>(
      "Show Bounding Boxes", property_prefix_,
      boost::bind( &ObjectRecognitionRvizUI::getShowBoundingBoxes, object_recognition_ui_ ),
      boost::bind( &ObjectRecognitionRvizUI::showBoundingBoxes, object_recognition_ui_, _1 ),
      parent_category_, this );
  //setPropertyHelpText(visual_enabled_property_, "Whether to show bounding boxes around all objects in the scene.");
}

}
